
class LimsError(Exception):
    """Base Error"""
    def __init__(self, message):
        self.message = message

class QueueArtifactsError(LimsError):
    """Raise when failing to route artifacts."""
    pass

class DuplicateSampleError(LimsError):
    """Raise when you excpect one sample, but find more. 
    Eg: Two pools in a run contain the same sample."""
    pass

class MissingArtifactError(LimsError):
    """Raise when searching for artifacts that don't exist.
    Eg: Found no artifact for sample X in process Y."""
    pass

class MissingUDFsError(LimsError):
    """Raise when searching for udfs that don't exist.
    Eg: Found no udf X on artifact Y."""
    pass





import click

def config(help: str = "config file for lims connection.") -> click.option:
    return click.option("-c", "--config", required=True, help=help)

def process(help: str = "Lims id for current Process.") -> click.option:
    return click.option("-p", "--process", required=True, help=help)

def log(help: str = "Path to log file.",) -> click.option:
    return click.option("-l", "--log", required=True, help=help)

def workflow_id(help: str = "Workflow id.") -> click.option:
    return click.option("-w", "--workflow-id", required=True, help=help)

def stage_id(help: str = "Stage id.") -> click.option:
    return click.option("-s", "--stage-id", required=True, help=help)

def udf(help: str = "UDF name",) -> click.option:
    return click.option("-u", "--udf", required=True, help=help)

def sample_qc_udf(help: str = "UDF name",) -> click.option:
    return click.option("-qc", "--sample-qc-udf", default="" ,required=False, help=help)

def artifact_udf(help: str = "UDF name",) -> click.option:
    return click.option("-audf", "--artifact-udf", required=True, help=help)

def sample_udf(help: str = "UDF name",) -> click.option:
    return click.option("-sudf", "--sample-udf", required=True, help=help)

def log(help: str = "Path to log file.",) -> click.option:
    return click.option("-l", "--log", required=True, help=help)

def file_placeholder(help: str = "File.",) -> click.option:
    return click.option("-f", "--file", required=True, help=help)

def input(
    help: str = "Use this flag if you run the script from a QC step.",
) -> click.option:
    return click.option(
        "-i", "--input", default=False, is_flag=True, help=help,
    )

def process_type(help: str = "Process type name.") -> click.option:
    return click.option(
        "-n", "--process-type", required=True, multiple=True, help=help,
    )



import string
import csv


def make_plate_file(file: str, rows: dict, headers: list, col_range: int=13, row_range: int=8, newline='\n', delimiter=','):
    """Creating a plate file.
    
    Arguments:
        file: file path
        headers: Header list. 
        rows: dict of rows
            keys: plate position: eg A1, B1, E4, H2...
            values: list of file rows. Same length as the headers list."""

    with open(file, "w", newline=newline) as hamilton_csv:
        wr = csv.writer(hamilton_csv, delimiter=delimiter)
        wr.writerow(headers)
        for col in range(1, col_range):
            for row in string.ascii_uppercase[0:row_range]:
                row = rows.get(f"{row}{col}")
                if row:
                    wr.writerow(row)



#!/usr/bin/env python
from cg_lims.get.files import get_lims_log_file
from cg_lims import options

from genologics.lims import Lims

from genologics.entities import Process

import pathlib
import logging
import click
import yaml

# commands
from cg_lims.EPPs import epps

@click.group(invoke_without_command=True)
@options.log()
@options.process()
@options.config()
@click.pass_context
def cli(ctx, log, process, config):

    with open(config) as file:
        config_data = yaml.load(file, Loader=yaml.FullLoader)
    lims = Lims(
        config_data["BASEURI"], config_data["USERNAME"], config_data["PASSWORD"]
    )

    log_path = pathlib.Path(log)
    if not log_path.is_file():
        log_path = get_lims_log_file(lims, log)
    logging.basicConfig(
        filename=str(log_path.absolute()), filemode="a", level=logging.INFO
    )
    process = Process(lims, id=process)
    ctx.ensure_object(dict)
    ctx.obj["lims"] = lims
    ctx.obj["process"] = process


cli.add_command(epps)



#!/usr/bin/env python

import click

# commands
from cg_lims.EPPs.files.make_kapa_csv import make_kapa_csv


@click.group(invoke_without_command=True)
@click.pass_context
def files(ctx):
    """Main entry point of file commands"""
    pass


files.add_command(make_kapa_csv)



from genologics.entities import Process, Artifact, Sample
from genologics.config import BASEURI
from genologics.lims import Lims

from operator import attrgetter
import sys
import pathlib
from typing import List


def get_lims_log_file(lims: Lims, file_id: str) -> pathlib.Path:
    """Searching for a log Artifact with file_id. 
    
    If the artifact is found, returning the path to the attached file. 
    Otherwise returning the file_id."""

    log_artifact = Artifact(lims, id=file_id)

    try:
        files = log_artifact.files
    except:
        files = None

    if files:
        server_adress = BASEURI.split(":")[1]
        file_path = files[0].content_location.split(server_adress)[1]
    else:
        file_path = file_id

    return pathlib.Path(file_path)



#!/usr/bin/env python

import click

# commands
from cg_lims.EPPs.move import move
from cg_lims.EPPs.files import files
from cg_lims.EPPs.udf import udf


@click.group(invoke_without_command=True)
@click.pass_context
def epps(ctx):
    """Main entry point of epp commands"""
    pass


epps.add_command(move)
epps.add_command(files)
epps.add_command(udf)



#!/usr/bin/env python

from cg_lims.get.artifacts import get_artifacts, get_latest_artifact
from cg_lims.files.make_csv import make_plate_file
from cg_lims import options
from genologics.lims import Lims
from genologics.entities import Process

import string
import logging
import click

LOG = logging.getLogger(__name__)


def get_file_rows(lims: Lims, amount_step: str, artifacts: list) -> dict:
    """Getting row data for hamilton file based on amount, reagent label and"""

    translate_amount = {
        10: {"Ligation Master Mix": "B", "PCR Plate": "Plate3"},
        50: {"Ligation Master Mix": "A", "PCR Plate": "Plate2"},
        250: {"Ligation Master Mix": "A", "PCR Plate": "Plate1"},
    }

    file_rows = {}
    for art in artifacts:
        sample_id = art.samples[0].id
        if art.reagent_labels:
            # Assuming one reagent label per artifact (reagent_labels is a list)
            reagent_label = art.reagent_labels[0]
            # A ragent label typically looks like this: 'A05 IDT_10nt_446 (AGCGTGACCT-CCATCCGAGT)'
            # Getting the index well:
            index_well_with_zero = reagent_label.split(" ")[0]
            # Picking out column and removing zeros by int():
            index_well_col = int(index_well_with_zero[1:])
            index_well_row = index_well_with_zero[0]
            index_well = f"{index_well_row}{index_well_col}"
        else:
            index_well = "-"
        well = art.location[1].replace(":", "")
        amount_artifact = get_latest_artifact(lims, sample_id, amount_step)
        amount = amount_artifact.udf.get("Amount needed (ng)")
        if amount <= 10:
            amount = 10
        mix_plate = translate_amount.get(amount)
        if mix_plate:
            file_rows[well] = [
                sample_id,
                well,
                mix_plate["Ligation Master Mix"],
                index_well,
                mix_plate["PCR Plate"],
            ]
            art.udf["Ligation Master Mix"] = mix_plate["Ligation Master Mix"]
            art.udf["PCR Plate"] = mix_plate["PCR Plate"]
            art.put()
        else:
            failed_samples.append(sample_id)
    return file_rows


@click.command()
@options.process_type(help="Amount step name.")
@options.file_placeholder(help="Hamilton file.")
@click.pass_context
def make_kapa_csv(ctx, file, process_type):
    """Script to make a csv file for hamilton."""

    process = ctx.obj["process"]
    lims = ctx.obj["lims"]
    artifacts = get_artifacts(process=process, input=False)
    file_rows = get_file_rows(lims, process_type, artifacts)
    headers = [
        "LIMS ID",
        "Sample Well",
        "Ligation Master Mix",
        "Index Well",
        "PCR Plate",
    ]
    file = f"{file}_KAPA_Hamilton.txt"
    make_plate_file(file, file_rows, headers)




from genologics.entities import Process, Artifact, Sample
from genologics.lims import Lims

from operator import attrgetter
from typing import List


from cg_lims.exceptions import QueueArtifactsError, MissingArtifactError



def get_sample_artifact(lims: Lims, sample: Sample) -> Artifact:
    """Returning the initial artifact related to a sample.
    Assuming first artifact is allways named sample.id + 'PA1."""

    return Artifact(lims, id=f"{sample.id}PA1")


def get_artifacts(process: Process, input: bool) -> List[Artifact]:
    """If inputs is True, returning all input analytes of the process,
    otherwise returning all output analytes of the process"""

    if input:
        artifacts = process.all_inputs(unique=True)
    else:
        artifacts = [a for a in process.all_outputs(unique=True) if a.type == "Analyte"]
    return artifacts


def filter_artifacts(artifacts: List[Artifact], udf: str, value) -> List[Artifact]:
    """Returning a list of only artifacts with udf==value"""

    return [a for a in artifacts if a.udf.get(udf) == value]


def get_latest_artifact(lims: Lims, sample_id: str, process_type: List[str]) -> Artifact:
    """Getting the most recently generated artifact by process_type and sample_id.

    Searching for all artifacts (Analytes) associated with <sample_id> that
    were produced by <process_type>. 
    
    Returning the artifact with latest parent_process.date_run. 
    If there are many such artifacts only one will be returned."""

    artifacts = lims.get_artifacts(
        samplelimsid=sample_id,
        type="Analyte",
        process_type=process_type,
    )
    if not artifacts:
        raise MissingArtifactError(
            message=f"Could not find a artifact with sample: {sample_id} generated by process: {' ,'.join(process_type)}"
        )
    sorted(artifacts, key=attrgetter("parent_process.date_run"))

    return artifacts[-1]






#!/usr/bin/env python

from setuptools import setup, find_packages

try:
    with open("requirements.txt", "r") as f:
        install_requires = [x.strip() for x in f.readlines()]
except IOError:
    install_requires = []


setup(name='cg_lims',
        version='1.0',
        description='',
        author='Maya Brandi',
        author_email='maya.brandi@scilifelab.se',
        packages=find_packages(),
        include_package_data=True,
        entry_points={
        'console_scripts': ['lims=cg_lims.commands:cli'],
    },

     )



from genologics.entities import Process, Sample
from typing import List



def get_process_samples(process: Process) -> List[Sample]:
    """Get all samples in a process"""

    all_samples = []
    for art in process.all_inputs():
        all_samples += art.samples

    return list(set(all_samples))
